#
# wls_install::opatch
#
# @summary This defined type allows you to apply an OPatch on a Middleware home or a Oracle product home.
# Using this defined type you can ensure your WebLogic installation is always at the required patchlevel. Here is an example on how you could use this:
# 
# ```puppet
# ora_install::opatch {'16175470':
#   ensure                  => "present",
#   oracle_product_home_dir => "/opt/oracle/middleware12c",
#   jdk_home_dir            => "/usr/java/jdk1.7.0_45",
#   patch_id                => "16175470",
#   patch_file              => "p16175470_121200_Generic.zip",
#   os_user                 => "oracle",
#   os_group                => "dba",
#   download_dir            => "/data/install",
#   source                  => "/vagrant",
#   log_output              => false,
# }
# ```
#
# @param [Enum[present, absent]] ensure
#    State to obtain.
#    The ensure attribute can be one of two values:
#    - present
#    - absent
#    When you specify `present`, Puppet will make sure the resource is available with all specified options and properties.
#    When the resource is already available(installed), and all attributes are as the are specified, Puppet will do nothing.
#    When you specify `absent`, Puppet will remove the resource if it is available. If it is not installed, Puppet will do nothing.
#
# @param [Stdlib::Absolutepath] oracle_product_home_dir
#    Oracle product home.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [Integer] patch_id
#    Id of the patch to install.
#
# @param [String[1]] patch_file
#    Patch file to use.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] source
#    Source for the operation.
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Optional[Variant[Boolean, Enum['on_failure']]]] log_output
#    **DEPRECATED** This parameter is replaced by 'logoutput' and will be removed in a future release.
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Optional[Boolean]] remote_file
#    **DEPRECATED**
#    This parameter is not being used anymore and will be removed in a future release.
#    `true`.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::opatch(
  Enum[
    present,
    absent]            $ensure                  = present,
  Stdlib::Absolutepath $oracle_product_home_dir = undef,
  Stdlib::Absolutepath $jdk_home_dir            = lookup({name => 'wls_jdk_home_dir', default_value => undef}),
  Integer              $patch_id                = undef,
  String[1]            $patch_file              = undef,
  String[1]            $os_user                 = lookup({name => 'wls_os_user', default_value => undef}),
  String[1]            $os_group                = lookup({name => 'wls_os_group', default_value => undef}),
  Stdlib::Absolutepath $download_dir            = lookup({name => 'wls_download_dir', default_value => undef}),
  String[1]            $source                  = lookup({name => 'wls_source', default_value => undef}),
  Variant[
    Boolean,
    Enum[
      'on_failure'
    ]
  ]                    $logoutput               = lookup({name => 'logoutput', default_value => 'on_failure'}),
  Optional[
    Variant[
      Boolean,
      Enum[
        'on_failure'
      ]
    ]
  ]                    $log_output              = undef, # DEPRECATED
  Optional[Boolean]    $remote_file             = undef, # DEPRECATED
)
{
  if ( $log_output in [true,false,'on_failure'] ) {
    deprecation (log_output, "Parameter 'log_output' is deprecated and is replaced by 'logoutput'. It will be removed in a future release")
  }
  if ( $remote_file in [true,false] ) {
    deprecation (remote_file, "Parameter 'remote_file' is deprecated and will be removed in a future release")
  }

  if ( $source ) {
    $mount_point = $source
  } else {
    $mount_point = 'puppet:///modules/wls_install/'
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  if ( $ensure == present ) {
    archive { "${download_dir}/${patch_file}":
      ensure  => present,
      cleanup => false,
      extract => false,
      source  => "${mount_point}/${patch_file}",
      user    => $os_user,
      group   => $os_group,
      before  => Wls_opatch["${oracle_product_home_dir}:${patch_id}"],
    }
    $disk1_file = "${download_dir}/${patch_file}"
  } else {
    $disk1_file = undef
  }

  case $::kernel {
    'Linux', 'AIX': {
      $ora_inst_path = '/etc'
    }
    'SunOS': {
      $ora_inst_path = '/var/opt/oracle'
    }
    default: {
      fail("Unrecognized operating system ${::kernel}, please use it on a Linux host")
    }
  }

  wls_opatch{ "${oracle_product_home_dir}:${patch_id}":
    ensure       => $ensure,
    os_user      => $os_user,
    source       => $disk1_file,
    jdk_home_dir => $jdk_home_dir,
    orainst_dir  => $ora_inst_path,
    tmp_dir      => $download_dir,
  }
}
